@@ -39,14 +39,20 @@ extern int preserve_times;
 
 static char **local_exclude_list = NULL;
 
+static void clean_fname(char *name);
+
+
 /*
   This function is used to check if a file should be included/excluded
   from the list of files based on its name and type etc
  */
 static int match_file_name(char *fname,struct stat *st)
 {
-  if (check_exclude(fname,local_exclude_list)) 
+  if (check_exclude(fname,local_exclude_list)) {
+    if (verbose > 2)
+      fprintf(stderr,"excluding file %s\n",fname);
     return 0;
+  }
   return 1;
 }
 
@@ -245,8 +251,10 @@ struct file_list *send_file_list(int f,int recurse,int argc,char *argv[])
   char dbuf[MAXPATHLEN];
   struct file_list *flist;
 
-  if (verbose && recurse)
+  if (verbose && recurse) {
     fprintf(am_server?stderr:stdout,"building file list ... ");
+    fflush(am_server?stderr:stdout);
+  }
 
   flist = (struct file_list *)malloc(sizeof(flist[0]));
   if (!flist) out_of_memory("send_file_list");
@@ -438,44 +446,66 @@ int flist_find(struct file_list *flist,struct file_struct *f)
   return -1;
 }
 
+
+static void clean_fname(char *name)
+{
+  char *p;
+  int l;
+  int modified = 1;
+
+  if (!name) return;
+
+  while (modified) {
+    modified = 0;
+
+    if ((p=strstr(name,"/./"))) {
+      modified = 1;
+      while (*p) {
+	p[0] = p[2];
+	p++;
+      }
+    }
+
+    if ((p=strstr(name,"//"))) {
+      modified = 1;
+      while (*p) {
+	p[0] = p[1];
+	p++;
+      }
+    }
+
+    if (strncmp(p=name,"./",2) == 0) {      
+      modified = 1;
+      while (*p) {
+	p[0] = p[2];
+	p++;
+      }
+    }
+
+    l = strlen(p=name);
+    if (l > 1 && p[l-1] == '/') {
+      modified = 1;
+      p[l-1] = 0;
+    }
+  }
+}
+
+
 /*
  * This routine ensures we don't have any duplicate names in our file list.
  * duplicate names can cause corruption because of the pipelining 
  */
 void clean_flist(struct file_list *flist)
 {
-  int i,l;
-  char *p;
+  int i;
 
   if (!flist || flist->count == 0) 
     return;
   
   for (i=0;i<flist->count;i++) {
-    if (!flist->files[i].name) continue;
-    if ((p=strstr(flist->files[i].name,"/./"))) {
-      while (*p) {
-	p[0] = p[2];
-	p++;
-      }
-    }
-    if ((p=strstr(flist->files[i].name,"//"))) {
-      while (*p) {
-	p[0] = p[1];
-	p++;
-      }
-    }
-    if (strncmp(p=flist->files[i].name,"./",2) == 0) {      
-      while (*p) {
-	p[0] = p[2];
-	p++;
-      }
-    }
-    l = strlen(p=flist->files[i].name);
-    if (l > 1 && p[l-1] == '/')
-      p[l-1] = 0;
+    clean_fname(flist->files[i].name);
   }
-  
-    
+      
   qsort(flist->files,flist->count,
 	sizeof(flist->files[0]),
 	(int (*)())flist_compare);
